@@ -6,16 +6,11 @@ class Setup:
     INIT_SCRIPT_PATH = 'jal_init.sql'
     UPDATES_PATH = 'updates'
     UPDATE_PREFIX = 'jal_delta_'
-    TARGET_SCHEMA = 17
+    TARGET_SCHEMA = 22
     CALC_TOLERANCE = 1e-10
     DISP_TOLERANCE = 1e-4
 
 
-class ColumnWidth:
-    STRETCH = -1
-    FOR_DATETIME = -2
-
-
 class BookAccount:  # PREDEFINED BOOK ACCOUNTS
     Costs = 1
     Incomes = 2
@@ -60,6 +55,16 @@ class CorporateAction:   # CORPORATE ACTIONS FOR ASSETS
     StockDividend = 5
 
 
+class PredefindedAccountType:
+    Case = 1
+    Bank = 2
+    Card = 3
+    Investment = 4
+    Savings = 5
+    Loans = 6
+    eWallet = 7
+
+
 class PredefinedCategory:
     Income = 1
     Spending = 2
