@@ -5,7 +5,7 @@ import json
 import logging
 import pandas as pd
 from PySide2.QtWidgets import QStyledItemDelegate
-from jal.ui_custom.reference_selector import CategorySelector, TagSelector
+from jal.widgets.reference_selector import CategorySelector, TagSelector
 from jal.constants import CustomColor
 from jal.db.helpers import get_category_name
 try:
@@ -19,7 +19,7 @@ from PySide2.QtCore import Qt, Slot, Signal, QDateTime, QBuffer, QThread, QAbstr
 from PySide2.QtWidgets import QApplication, QDialog, QFileDialog, QHeaderView
 # This QCamera staff ran good on Windows but didn't fly on Linux from the box until 'cheese' installation
 from PySide2.QtMultimedia import QCameraInfo, QCamera, QCameraImageCapture, QVideoFrame
-from jal.ui_custom.helpers import g_tr, dependency_present
+from jal.widgets.helpers import g_tr, dependency_present
 from jal.db.helpers import executeSQL, readSQL
 from jal.data_import.slips_tax import SlipsTaxAPI
 from jal.ui.ui_slip_import_dlg import Ui_ImportSlipDlg
@@ -29,19 +29,15 @@ from jal.data_import.category_recognizer import recognize_categories
 #-----------------------------------------------------------------------------------------------------------------------
 # Custom model to display and edit slip lines
 class PandasLinesModel(QAbstractTableModel):
-    def __init__(self, data, db):
+    def __init__(self, data):
         QAbstractTableModel.__init__(self)
         self._data = data
-        self._db = db
 
     def flags(self, index):
         if not index.isValid():
             return Qt.ItemIsEnabled
         return super().flags(index) | Qt.ItemIsEditable
 
-    def database(self):
-        return self._db
-
     def rowCount(self, parent=None):
         return self._data.shape[0]
 
@@ -86,7 +82,7 @@ class SlipLinesDelegate(QStyledItemDelegate):
             text = model.data(index, Qt.DisplayRole)
             painter.drawText(option.rect, Qt.AlignLeft | Qt.AlignVCenter, text)
         if index.column() == 1:
-            text = get_category_name(index.model().database(), int(model.data(index, Qt.DisplayRole)))
+            text = get_category_name(int(model.data(index, Qt.DisplayRole)))
             confidence = model.data(index.siblingAtColumn(2), Qt.DisplayRole)
             if confidence > 0.75:
                 painter.fillRect(option.rect, CustomColor.LightGreen)
@@ -108,11 +104,9 @@ class SlipLinesDelegate(QStyledItemDelegate):
     def createEditor(self, aParent, option, index):
         if index.column() == 1:
             category_selector = CategorySelector(aParent)
-            category_selector.init_db(index.model().database())
             return category_selector
         if index.column() == 3:
             tag_selector = TagSelector(aParent)
-            tag_selector.init_db(index.model().database())
             return tag_selector
 
     def setModelData(self, editor, model, index):
@@ -135,13 +129,12 @@ class ImportSlipDialog(QDialog, Ui_ImportSlipDlg):
     QR_pattern = "^t=(.*)&s=(.*)&fn=(.*)&i=(.*)&fp=(.*)&n=(.*)$"
     timestamp_patterns = ['yyyyMMddTHHmm', 'yyyyMMddTHHmmss', 'yyyy-MM-ddTHH:mm', 'yyyy-MM-ddTHH:mm:ss']
 
-    def __init__(self, parent, db):
+    def __init__(self, parent):
         QDialog.__init__(self, parent=parent)
         self.setupUi(self)
         self.initUi()
-        self.db=db
         self.model = None
-        self.delegates = []
+        self.delegate = []
 
         self.CameraGroup.setVisible(False)
         self.cameraActive = False
@@ -152,9 +145,7 @@ class ImportSlipDialog(QDialog, Ui_ImportSlipDlg):
         self.slip_json = None
         self.slip_lines = None
 
-        self.slipsAPI = SlipsTaxAPI(self.db)
-        self.AccountEdit.init_db(self.db)
-        self.PeerEdit.init_db(self.db)
+        self.slipsAPI = SlipsTaxAPI()
 
         self.qr_data_available.connect(self.parseQRdata)
         self.LoadQRfromFileBtn.clicked.connect(self.loadFileQR)
@@ -399,10 +390,10 @@ class ImportSlipDialog(QDialog, Ui_ImportSlipDlg):
         self.slip_lines['tag'] = None
         self.slip_lines = self.slip_lines[['name', 'category', 'confidence', 'tag', 'sum']]
 
-        self.model = PandasLinesModel(self.slip_lines, self.db)
+        self.model = PandasLinesModel(self.slip_lines)
         self.LinesTableView.setModel(self.model)
 
-        self.delegates = []    # FIXME - we don't need to keep a list, we may create one delegate and assign it to every column
+        self.delegate = SlipLinesDelegate(self.LinesTableView)
         for column in range(self.model.columnCount()):
             if column == 0:
                 self.LinesTableView.horizontalHeader().setSectionResizeMode(column, QHeaderView.Stretch)
@@ -412,8 +403,7 @@ class ImportSlipDialog(QDialog, Ui_ImportSlipDlg):
                 self.LinesTableView.setColumnHidden(column, True)
             else:
                 self.LinesTableView.setColumnWidth(column, 100)
-            self.delegates.append(SlipLinesDelegate(self.LinesTableView))
-            self.LinesTableView.setItemDelegateForColumn(column, self.delegates[-1])
+            self.LinesTableView.setItemDelegateForColumn(column, self.delegate)
         font = self.LinesTableView.horizontalHeader().font()
         font.setBold(True)
         self.LinesTableView.horizontalHeader().setFont(font)
@@ -431,25 +421,24 @@ class ImportSlipDialog(QDialog, Ui_ImportSlipDlg):
             logging.warning(g_tr('ImportSlipDialog', "Not possible to import slip: some categories are not set"))
             return
 
-        query = executeSQL(self.db, "INSERT INTO actions (timestamp, account_id, peer_id) "
-                                     "VALUES (:timestamp, :account_id, :peer_id)",
-                            [(":timestamp", self.SlipDateTime.dateTime().toSecsSinceEpoch()),
-                             (":account_id", self.AccountEdit.selected_id),
-                             (":peer_id", self.PeerEdit.selected_id)])
+        query = executeSQL("INSERT INTO actions (timestamp, account_id, peer_id) "
+                           "VALUES (:timestamp, :account_id, :peer_id)",
+                           [(":timestamp", self.SlipDateTime.dateTime().toSecsSinceEpoch()),
+                            (":account_id", self.AccountEdit.selected_id),
+                            (":peer_id", self.PeerEdit.selected_id)])
         pid = query.lastInsertId()
         # update mappings
-        _ = executeSQL(self.db, "INSERT INTO map_peer (value, mapped_to) VALUES (:peer_name, :peer_id)",
+        _ = executeSQL("INSERT INTO map_peer (value, mapped_to) VALUES (:peer_name, :peer_id)",
                        [(":peer_name", self.SlipShopName.text()), (":peer_id", self.PeerEdit.selected_id)])
 
         for index, row in self.slip_lines.iterrows():
-            _ = executeSQL(self.db, "INSERT INTO action_details (pid, category_id, tag_id, sum, note) "
-                                    "VALUES (:pid, :category_id, :tag_id, :amount, :note)",
+            _ = executeSQL("INSERT INTO action_details (pid, category_id, tag_id, sum, note) "
+                           "VALUES (:pid, :category_id, :tag_id, :amount, :note)",
                            [(":pid", pid), (":category_id", row['category']), (":tag_id", row['tag']),
                             (":amount", row['sum']), (":note", row['name'])])
             # update mappings
-            _ = executeSQL(self.db, "INSERT INTO map_category (value, mapped_to) VALUES (:item_name, :category_id)",
-                           [(":item_name", row['name']), (":category_id", row['category'])])
-        self.db.commit()
+            _ = executeSQL("INSERT INTO map_category (value, mapped_to) VALUES (:item_name, :category_id)",
+                           [(":item_name", row['name']), (":category_id", row['category'])], commit=True)
         self.clearSlipData()
 
     def clearSlipData(self):
@@ -461,11 +450,11 @@ class ImportSlipDialog(QDialog, Ui_ImportSlipDlg):
         self.initUi()
 
     def match_shop_name(self, shop_name):
-        return readSQL(self.db, "SELECT mapped_to FROM map_peer WHERE value=:shop_name",
+        return readSQL("SELECT mapped_to FROM map_peer WHERE value=:shop_name",
                        [(":shop_name", shop_name)])
 
     @Slot()
     def recognizeCategories(self):
         self.slip_lines['category'], self.slip_lines['confidence'] = \
-            recognize_categories(self.db, self.slip_lines['name'].tolist())
+            recognize_categories(self.slip_lines['name'].tolist())
         self.model.dataChanged.emit(None, None)  # refresh full view
\ No newline at end of file
