@@ -5,7 +5,7 @@ from PySide2.QtSql import QSql, QSqlDatabase, QSqlQuery
 from PySide2.QtWidgets import QMessageBox
 from jal.constants import Setup
 from jal.db.settings import JalSettings
-from jal.ui_custom.helpers import g_tr
+from jal.widgets.helpers import g_tr
 
 
 # No translation of the file because these routines might be used before QApplication initialization
@@ -39,11 +39,34 @@ class LedgerInitError:
 def get_dbfilename(app_path):
     return app_path + Setup.DB_PATH
 
+
+# -------------------------------------------------------------------------------------------------------------------
+# This function returns SQLite connection used by JAL or fails with RuntimeError exception
+def db_connection():
+    db = QSqlDatabase.database(Setup.DB_CONNECTION)
+    if not db.isValid():
+        raise RuntimeError(f"DB connection '{Setup.DB_CONNECTION}' is invalid")
+    if not db.isOpen():
+        logging.fatal(f"DB connection '{Setup.DB_CONNECTION}' is not open")
+    return db
+
+
+# -------------------------------------------------------------------------------------------------------------------
+def db_triggers_disable():
+    _ = executeSQL("UPDATE settings SET value=0 WHERE name='TriggersEnabled'", commit=True)
+
+# -------------------------------------------------------------------------------------------------------------------
+def db_triggers_enable():
+    _ = executeSQL("UPDATE settings SET value=1 WHERE name='TriggersEnabled'", commit=True)
+
 # -------------------------------------------------------------------------------------------------------------------
 # prepares SQL query from given sql_text
 # params_list is a list of tuples (":param", value) which are used to prepare SQL query
+# Current transactin will be commited if 'commit' set to true
+# Parameter 'forward_only' may be used for optimization
 # return value - QSqlQuery object (to allow iteration through result)
-def executeSQL(db, sql_text, params = [], forward_only = True):
+def executeSQL(sql_text, params=[], forward_only=True, commit=False):
+    db = db_connection()
     query = QSqlQuery(db)
     query.setForwardOnly(forward_only)
     if not query.prepare(sql_text):
@@ -54,16 +77,18 @@ def executeSQL(db, sql_text, params = [], forward_only = True):
     if not query.exec_():
         logging.error(f"SQL exec: '{query.lastError().text()}' for query '{sql_text}' with params '{params}'")
         return None
+    if commit:
+        db.commit()
     return query
 
 
 # -------------------------------------------------------------------------------------------------------------------
 # the same as executeSQL() but after query execution it takes first line of it
 # and packs all field values in a list to return
-def readSQL(db, sql_text, params=None, named=False):
+def readSQL(sql_text, params=None, named=False):
     if params is None:
         params = []
-    query = QSqlQuery(db)
+    query = QSqlQuery(db_connection())
     query.setForwardOnly(True)
     if not query.prepare(sql_text):
         logging.error(f"SQL prep: '{query.lastError().text()}' for query '{sql_text}' with params '{params}'")
@@ -103,11 +128,11 @@ def readSQLrecord(query, named=False):
 #    if not - it will initialize DB with help of SQL-script
 # 2) checks that DB looks like a valid one:
 #    if schema version is invalid it will close DB
-# Returns: db hanlder, LedgerInitError(code = 0 if db initialzied successfully)
+# Returns: LedgerInitError(code = 0 if db was initialized successfully)
 def init_and_check_db(db_path):
     db = QSqlDatabase.addDatabase("QSQLITE", Setup.DB_CONNECTION)
     if not db.isValid():
-        return None, LedgerInitError(LedgerInitError.DbDriverFailure)
+        return LedgerInitError(LedgerInitError.DbDriverFailure)
     db.setDatabaseName(get_dbfilename(db_path))
     db.open()
     tables = db.tables(QSql.Tables)
@@ -116,19 +141,20 @@ def init_and_check_db(db_path):
         connection_name = db.connectionName()
         init_db_from_sql(get_dbfilename(db_path), db_path + Setup.INIT_SCRIPT_PATH)
         QSqlDatabase.removeDatabase(connection_name)
-        return None, LedgerInitError(LedgerInitError.EmptyDbInitialized)
+        return LedgerInitError(LedgerInitError.EmptyDbInitialized)
 
     schema_version = JalSettings().getValue('SchemaVersion')
     if schema_version < Setup.TARGET_SCHEMA:
         db.close()
-        return None, LedgerInitError(LedgerInitError.OutdatedDbSchema)
+        return LedgerInitError(LedgerInitError.OutdatedDbSchema)
     elif schema_version > Setup.TARGET_SCHEMA:
         db.close()
-        return None, LedgerInitError(LedgerInitError.NewerDbSchema)
+        return LedgerInitError(LedgerInitError.NewerDbSchema)
 
-    _ = executeSQL(db, "PRAGMA foreign_keys = ON")
+    _ = executeSQL("PRAGMA foreign_keys = ON")
+    db_triggers_enable()
 
-    return db, LedgerInitError(LedgerInitError.DbInitSuccess)
+    return LedgerInitError(LedgerInitError.DbInitSuccess)
 
 
 # -------------------------------------------------------------------------------------------------------------------
@@ -173,35 +199,31 @@ def update_db_schema(db_path):
 
 
 # -------------------------------------------------------------------------------------------------------------------
-def get_language(db):   # TODO Change to make it via JalSettings instead of readSQL
-    language = ''
-    if db:
-        language = readSQL(db, "SELECT l.language FROM settings AS s "
-                               "LEFT JOIN languages AS l ON s.value=l.id WHERE s.name='Language'")
+def get_language():   # TODO Change to make it via JalSettings instead of readSQL
+    language = readSQL("SELECT l.language FROM settings AS s "
+                       "LEFT JOIN languages AS l ON s.value=l.id WHERE s.name='Language'")
     language = 'us' if language == '' else language
     return language
 
 # -------------------------------------------------------------------------------------------------------------------
-def get_field_by_id_from_table(db, table_name, field_name, id):
+def get_field_by_id_from_table(table_name, field_name, id):
     SQL = f"SELECT t.{field_name} FROM {table_name} AS t WHERE t.id = :id"
-    return readSQL(db, SQL, [(":id", id)])
+    return readSQL(SQL, [(":id", id)])
 
 # -------------------------------------------------------------------------------------------------------------------
-def get_category_name(db, category_id):
-    return readSQL(db, "SELECT c.name FROM categories AS c WHERE c.id=:category_id", [(":category_id", category_id)])
+def get_category_name(category_id):
+    return readSQL("SELECT c.name FROM categories AS c WHERE c.id=:category_id", [(":category_id", category_id)])
 
 # -------------------------------------------------------------------------------------------------------------------
-def get_account_name(db, account_id):
-    return readSQL(db, "SELECT name FROM accounts WHERE id=:account_id", [(":account_id", account_id)])
+def get_account_name(account_id):
+    return readSQL("SELECT name FROM accounts WHERE id=:account_id", [(":account_id", account_id)])
 
-def get_account_currency(db, account_id):
-    return readSQL(db, "SELECT currency_id FROM accounts WHERE id=:account_id", [(":account_id", account_id)])
 # -------------------------------------------------------------------------------------------------------------------
-def get_country_by_code(db, country_code):
-    id = readSQL(db, "SELECT id FROM countries WHERE code=:code", [(":code", country_code)])
+def get_country_by_code(country_code):
+    id = readSQL("SELECT id FROM countries WHERE code=:code", [(":code", country_code)])
 
     if id is None:
-        query = executeSQL(db, "INSERT INTO countries(name, code, tax_treaty) VALUES (:name, :code, 0)",
+        query = executeSQL("INSERT INTO countries(name, code, tax_treaty) VALUES (:name, :code, 0)",
                            [(":name", "Country_" + country_code), (":code", country_code)])
         id = query.lastInsertId()
         logging.warning(g_tr('DB', "New country added (set Tax Treaty in Data->Countries menu): ")
@@ -212,27 +234,23 @@ def get_country_by_code(db, country_code):
 # Function sets asset country if asset.country_id is 0
 # Shows warning and sets new asset country if asset.country_id was different before
 # Does nothing if asset country had already the same value
-def update_asset_country(db, asset_id, country_id):
-    id = readSQL(db, "SELECT country_id FROM assets WHERE id=:asset_id", [(":asset_id", asset_id)])
+def update_asset_country(asset_id, country_id):
+    id = readSQL("SELECT country_id FROM assets WHERE id=:asset_id", [(":asset_id", asset_id)])
     if id == country_id:
         return
-    _ = executeSQL(db, "UPDATE assets SET country_id=:country_id WHERE id=:asset_id",
+    _ = executeSQL("UPDATE assets SET country_id=:country_id WHERE id=:asset_id",
                    [(":asset_id", asset_id), (":country_id", country_id)])
     if id == 0:
         return
-    old_country = readSQL(db, "SELECT name FROM countries WHERE id=:country_id", [(":country_id", id)])
-    new_country = readSQL(db, "SELECT name FROM countries WHERE id=:country_id", [(":country_id", country_id)])
-    asset_name = readSQL(db, "SELECT name FROM assets WHERE id=:asset_id", [(":country_id", asset_id)])
+    old_country = readSQL("SELECT name FROM countries WHERE id=:country_id", [(":country_id", id)])
+    new_country = readSQL("SELECT name FROM countries WHERE id=:country_id", [(":country_id", country_id)])
+    asset_name = readSQL("SELECT name FROM assets WHERE id=:asset_id", [(":country_id", asset_id)])
     logging.warning(g_tr('DB', "Country was changed for asset ")+ f"{asset_name}: f{old_country} -> {new_country}")
 
 # -------------------------------------------------------------------------------------------------------------------
-def account_last_date(db, account_number):
-    last_timestamp = readSQL(db, "SELECT MAX(o.timestamp) FROM all_operations AS o "
-                                 "LEFT JOIN accounts AS a ON o.account_id=a.id WHERE a.number=:account_number",
+def account_last_date(account_number):
+    last_timestamp = readSQL("SELECT MAX(o.timestamp) FROM all_operations AS o "
+                             "LEFT JOIN accounts AS a ON o.account_id=a.id WHERE a.number=:account_number",
                              [(":account_number", account_number)])
     last_timestamp = 0 if last_timestamp == '' else last_timestamp
     return last_timestamp
-
-# -------------------------------------------------------------------------------------------------------------------
-def get_asset_name(db, asset_id):
-    return readSQL(db, "SELECT name FROM assets WHERE id=:asset_id", [(":asset_id", asset_id)])
\ No newline at end of file
